@@ -19,16 +19,16 @@ show_notification (void* data)
                                        "network service discovery. The service "
                                        "has been disabled."
 				       ),
-				     GTK_STOCK_DIALOG_INFO,
-				     NULL);
+				     GTK_STOCK_DIALOG_INFO);
 	notify_notification_set_timeout (n, 60000);
 	notify_notification_show (n, NULL);
+	g_object_unref (n);
 
 	return FALSE;
 }
 
 gboolean
-avahi_disabled_check()
+avahi_disabled_check(void)
 {
         if (g_file_test (UNICAST_LOCAL_AVAHI_FILE, G_FILE_TEST_EXISTS)) {
                 /* Show the notification, after a delay so it doesn't look ugly
