@@ -33,6 +33,7 @@
 #include <limits.h>
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <glib-unix.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gio/gio.h>
@@ -66,8 +67,8 @@ gboolean update_timer_finished(gpointer data);
 // force startup even if the user is not in the sudo/admin group
 static gboolean FORCE_START=FALSE;
 
-// force gksu for all menu actions
-static gboolean FORCE_GKSU=FALSE;
+// force pkexec for all menu actions
+static gboolean FORCE_PKEXEC=FALSE;
 
 // delay on startup (to make session startup faster)
 static int STARTUP_DELAY=1;
@@ -79,12 +80,12 @@ gboolean DEVEL_RELEASE=FALSE;
 static gboolean FORCE_RELEASE_CHECK=FALSE;
 
 // global debug options
-int HOOK_DEBUG = 0;
-int UPDATE_DEBUG = 0;
-int INOTIFY_DEBUG = 0;
-int UEVENT_DEBUG = 0;
-int RELEASE_DEBUG = 0;
-int MISC_DEBUG = 0;
+static gboolean HOOK_DEBUG = FALSE;
+static gboolean UPDATE_DEBUG = FALSE;
+static gboolean INOTIFY_DEBUG = FALSE;
+static gboolean UEVENT_DEBUG = FALSE;
+static gboolean RELEASE_DEBUG = FALSE;
+static gboolean MISC_DEBUG = FALSE;
 
 // logging stuff
 static void debug_log_handler (const gchar   *log_domain,
@@ -93,26 +94,26 @@ static void debug_log_handler (const gchar   *log_domain,
                               gpointer       user_data)
 {
    if (log_domain && strcmp(log_domain, "hooks") == 0) {
-      if (HOOK_DEBUG > 0)
+      if (HOOK_DEBUG)
          g_log_default_handler (log_domain, log_level, message, user_data);
    }
    else if (log_domain && strcmp(log_domain, "update") == 0) {
-      if (UPDATE_DEBUG > 0)
+      if (UPDATE_DEBUG)
          g_log_default_handler (log_domain, log_level, message, user_data);
    }
    else if (log_domain && strcmp(log_domain, "inotify") == 0) {
-      if (INOTIFY_DEBUG > 0) 
+      if (INOTIFY_DEBUG) 
           g_log_default_handler (log_domain, log_level, message, user_data);
    }
    else if (log_domain && strcmp(log_domain, "uevent") == 0) {
-      if (UEVENT_DEBUG > 0)
+      if (UEVENT_DEBUG)
          g_log_default_handler (log_domain, log_level, message, user_data);
    }
    else if (log_domain && strcmp(log_domain, "release") == 0) {
-      if (RELEASE_DEBUG > 0)
+      if (RELEASE_DEBUG)
          g_log_default_handler (log_domain, log_level, message, user_data);
    }
-   else if (MISC_DEBUG > 0)
+   else if (MISC_DEBUG)
       g_log_default_handler (log_domain, log_level, message, user_data);
 }
 
@@ -135,7 +136,7 @@ static void start_next_script(GPid pid,
         gchar*       full_path = remaining_scripts_to_run->data;
         gchar       *argv[] = { full_path, NULL };
 
-        g_debug_inotify ("executing: %s\n", full_path);
+        g_debug_inotify ("executing: %s", full_path);
         if (g_file_test(full_path, G_FILE_TEST_IS_EXECUTABLE ))
             ret = g_spawn_async("/", argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
         remaining_scripts_to_run = g_slist_remove(remaining_scripts_to_run, full_path);
@@ -151,16 +152,16 @@ static void start_next_script(GPid pid,
 
 }
 
-void invoke(const gchar *cmd, const gchar *desktop, gboolean with_gksu)
+void invoke(const gchar *cmd, const gchar *desktop, gboolean with_pkexec)
 {
    GdkAppLaunchContext *context;
    GAppInfo *appinfo;
    GError *error = NULL;
    static GtkWidget *w = NULL;
 
-   // gksu
-   if(with_gksu || FORCE_GKSU) {
-      invoke_with_gksu(cmd, desktop, FALSE);
+   // pkexec
+   if(with_pkexec || FORCE_PKEXEC) {
+      invoke_with_pkexec(cmd);
       return;
    }
 
@@ -171,7 +172,7 @@ void invoke(const gchar *cmd, const gchar *desktop, gboolean with_gksu)
    }
 
    // normal launch
-   context = gdk_app_launch_context_new ();
+   context = gdk_display_get_app_launch_context (gdk_display_get_default ());
    guint32 timestamp =  gdk_x11_get_server_time (gtk_widget_get_window (w));
    appinfo = g_app_info_create_from_commandline(cmd, 
 						cmd, 
@@ -179,43 +180,31 @@ void invoke(const gchar *cmd, const gchar *desktop, gboolean with_gksu)
 						&error);
    gdk_app_launch_context_set_timestamp (context, timestamp);
    if (!g_app_info_launch (appinfo, NULL, (GAppLaunchContext*)context, &error))
-      g_warning ("Launching failed: %s\n", error->message);
+      g_warning ("Launching failed: %s", error->message);
    g_object_unref (context);
    g_object_unref (appinfo);
 
 }
 
 void
-invoke_with_gksu(const gchar *cmd, const gchar *descr, gboolean whole_message)
+invoke_with_pkexec(const gchar *cmd)
 {
-        g_debug("invoke_update_manager ()\n");
-        gchar *argv[5];
-	argv[0] = "/usr/bin/gksu";
-	argv[1] = whole_message ? "--message" : "--desktop";
-	argv[2] = (gchar*)descr;
-	argv[3] = (gchar*)cmd;
-	argv[4] = NULL;
-
-	g_spawn_async (NULL, argv, NULL, 0, NULL, NULL, NULL, NULL);
+        g_debug("invoke_with_pkexec ()");
+        gchar *argv[3];
+        argv[0] = "/usr/bin/pkexec";
+        argv[1] = (gchar*)cmd;
+        argv[2] = NULL;
+        g_spawn_async (NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, NULL, NULL);
 }
 
 
-
-gboolean
-trayapplet_create (TrayApplet *un, char *name)
+static gboolean
+trayapplet_create (TrayApplet *ta, UpgradeNotifier *un, char *name)
 {
-        g_debug("trayicon_create()\n");
+        g_debug("trayapplet_create()");
 
-	/* setup widgets */
-	un->name = name;
-#ifdef HAVE_APP_INDICATOR
-        un->indicator = app_indicator_new (name,
-                                           name,
-                                           APP_INDICATOR_CATEGORY_SYSTEM_SERVICES);
-#else
-	un->tray_icon = gtk_status_icon_new_from_icon_name (name);
-	gtk_status_icon_set_visible (un->tray_icon, FALSE);
-#endif
+	ta->un = un;
+	ta->name = name;
 
 	return TRUE;
 }
@@ -232,7 +221,7 @@ trayapplet_create (TrayApplet *un, char *name)
   - /var/lib/dpkg/lock
   - /var/lib/apt/periodic/update-success-stamp
 */
-void
+static void
 monitor_cb(GFileMonitor *handle,
 	   GFile *monitor_f,
 	   GFile *other_monitor_f,
@@ -243,8 +232,8 @@ monitor_cb(GFileMonitor *handle,
 
    gchar *info_uri = g_file_get_path(monitor_f);
 #if 0
-   g_debug_inotify("info_uri: %s\n", info_uri);
-   g_debug_inotify("event_type: %i\n",event_type);
+   g_debug_inotify("info_uri: %s", info_uri);
+   g_debug_inotify("event_type: %i",event_type);
 #endif
 
    // we ignore lock file events because we can only get 
@@ -257,7 +246,7 @@ monitor_cb(GFileMonitor *handle,
       || g_str_has_prefix(info_uri,"/var/cache/apt/")
       || strcmp(info_uri,"/var/lib/dpkg/status") == 0) {
       g_debug_inotify("apt_get_running=TRUE");
-      un->apt_get_runing=TRUE;
+      un->apt_get_running=TRUE;
    } 
    if(strstr(info_uri, "/var/lib/update-notifier/dpkg-run-stamp") ||
       strstr(info_uri, "/var/lib/apt/periodic/update-success-stamp")) {
@@ -265,37 +254,37 @@ monitor_cb(GFileMonitor *handle,
       un->dpkg_was_run = TRUE;
    } 
    if(strstr(info_uri, REBOOT_FILE)) {
-      g_debug_inotify("reboot required\n");
+      g_debug_inotify("reboot required");
       un->reboot_pending = TRUE;
    }
    if(strstr(info_uri, HOOKS_DIR)) {
-      g_debug_inotify("new hook!\n");
+      g_debug_inotify("new hook!");
       un->hook_pending = TRUE;
    }
    if(strstr(info_uri, CRASHREPORT_DIR)) {
-      g_debug_inotify("crashreport found\n");
+      g_debug_inotify("crashreport found");
       un->crashreport_pending = TRUE;
    }
    if(strstr(info_uri, UNICAST_LOCAL_AVAHI_FILE)) {
-      g_debug_inotify("avahi disabled due to unicast .local domain\n");
+      g_debug_inotify("avahi disabled due to unicast .local domain");
       un->unicast_local_avahi_pending = TRUE;
    }
 }
 
 /*
- * We periodically check here what actions happend in this "time-slice". 
+ * We periodically check here what actions happened in this "time-slice".
  * This can be:
  * - dpkg_was_run=TRUE: set when apt wrote the "dpkg-run-stamp" file
- * - apt_get_runing: set when apt/dpkg activity is detected (in the 
- *                   lists-dir, archive-cache, or /var/lib/dpkg/status
+ * - apt_get_running: set when apt/dpkg activity is detected (in the
+ *                    lists-dir, archive-cache, or /var/lib/dpkg/status)
  * - hook_pending: we have new upgrade hoook information
  * - reboot_pending: we need to reboot
  * - crashreport_pending: we have a new crashreport
  * - unicast_local_avahi_pending: avahi got disabled due to a unicast .local domain
  *
  */
-gboolean file_monitor_periodic_check(gpointer data)
-
+static gboolean
+file_monitor_periodic_check(gpointer data)
 {
    UpgradeNotifier *un = (UpgradeNotifier *)data;
 
@@ -303,7 +292,7 @@ gboolean file_monitor_periodic_check(gpointer data)
    if((un->reboot == NULL) || (un->crashreport == NULL))
       return TRUE;
 
-   // DPkg::Post-Invoke has written a stamp file, that means a install/remove
+   // DPkg::Post-Invoke has written a stamp file, that means an install/remove
    // operation finished, we can show hooks/reboot notifications then
    if(un->dpkg_was_run) {
 
@@ -319,10 +308,11 @@ gboolean file_monitor_periodic_check(gpointer data)
         cache_changed_scripts = g_slist_insert_sorted(cache_changed_scripts,
                                  (gpointer)full_path, (GCompareFunc)g_strcmp0);
       }
+      g_dir_close(dir);
       // launch first script and queue others
       start_next_script(0, 0, cache_changed_scripts);
 
-      // any apt-get update  must be finished, otherwise 
+      // any apt-get update must be finished, otherwise
       // apt-get install wouldn't be finished
       update_apt_is_running(un->update, FALSE);
       if(un->update_finished_timer > 0) 
@@ -334,13 +324,13 @@ gboolean file_monitor_periodic_check(gpointer data)
 	 un->reboot_pending = FALSE;
       }
 
-      // apt must be finished when a new stamp file was writen, so we
-      // reset the apt_get_runing time-slice field because its not 
-      // important anymore (it finished runing)
+      // apt must be finished when a new stamp file was written, so we
+      // reset the apt_get_running time-slice field because its not
+      // important anymore (it finished running)
       //
       // This may leave a 5s race condition when a apt-get install finished
       // and something new (apt-get) was started
-      un->apt_get_runing = FALSE;
+      un->apt_get_running = FALSE;
       un->last_apt_action = 0;
    }
 
@@ -351,14 +341,14 @@ gboolean file_monitor_periodic_check(gpointer data)
       un->hook_pending = FALSE;
    }
 
-   // apt-get update/install or dpkg is runing (and updates files in 
-   // it's list/cache dir) or in /var/lib/dpkg/status
-   if(un->apt_get_runing) 
+   // apt-get update/install or dpkg is running (and updates files in
+   // its list/cache dir) or in /var/lib/dpkg/status
+   if(un->apt_get_running) 
       update_apt_is_running(un->update, TRUE);
 
    // update time information for apt/dpkg
    time_t now = time(NULL);
-   if(un->apt_get_runing) 
+   if(un->apt_get_running) 
       un->last_apt_action = now;
 
    // no apt operation for a long time
@@ -370,20 +360,20 @@ gboolean file_monitor_periodic_check(gpointer data)
    }
 
    if(un->crashreport_pending) {
-      g_debug("checking for valid crashreport now\n");
+      g_debug("checking for valid crashreport now");
       crashreport_check (un->crashreport);
       un->crashreport_pending = FALSE;
    }
 
    if(un->unicast_local_avahi_pending) {
-      g_debug("checking for disabled avahi due to unicast .local domain now\n");
+      g_debug("checking for disabled avahi due to unicast .local domain now");
       avahi_disabled_check ();
       un->unicast_local_avahi_pending = FALSE;
    }
 
    // reset the bitfields (for the next "time-slice")
    un->dpkg_was_run = FALSE;
-   un->apt_get_runing = FALSE;
+   un->apt_get_running = FALSE;
 
    return TRUE;
 }
@@ -391,29 +381,14 @@ gboolean file_monitor_periodic_check(gpointer data)
 
 
 
-/* u_abort seems like an internal error notification.
- * End user might not understand the message at all */
-void u_abort(gchar *msg)
-{
-   const char *fmt = "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n";
-   gtk_dialog_run(GTK_DIALOG(gtk_message_dialog_new_with_markup(NULL, 0,
-						     GTK_MESSAGE_ERROR,
-						     GTK_BUTTONS_CLOSE,
-						     fmt,
-						     _("Internal error"),
-						     msg)));
-   g_free(msg);
-   exit(1);
-}
-
 // FIXME: get the apt directories with apt-config or something
-gboolean 
+static gboolean 
 monitor_init(UpgradeNotifier *un)
 {
    int i;
    GFileMonitor *monitor_handle;
 
-   // monitor thise dirs
+   // monitor these dirs
    static const char *monitor_dirs[] = { 
       "/var/lib/apt/lists/", "/var/lib/apt/lists/partial/", 
       "/var/cache/apt/archives/", "/var/cache/apt/archives/partial/", 
@@ -427,7 +402,7 @@ monitor_init(UpgradeNotifier *un)
       if(monitor_handle)
 	 g_signal_connect(monitor_handle, "changed", (GCallback)monitor_cb, un);
       else
-	 g_warning("can not add '%s'\n", monitor_dirs[i]);
+	 g_warning("can not add '%s'", monitor_dirs[i]);
    }
 
    // and those files
@@ -445,7 +420,7 @@ monitor_init(UpgradeNotifier *un)
       if(monitor_handle)
 	 g_signal_connect(monitor_handle, "changed", (GCallback)monitor_cb, un);
       else
-	 g_warning("can not add '%s'\n", monitor_dirs[i]);
+	 g_warning("can not add '%s'", monitor_dirs[i]);
    }
 
    g_timeout_add (TIMEOUT_FAM, (GSourceFunc)file_monitor_periodic_check, un);
@@ -462,46 +437,44 @@ tray_icons_init(UpgradeNotifier *un)
 {
    //g_debug_inotify("tray_icons_init");
 
-   /* new upates tray icon */
+   /* new updates tray icon */
    un->update = g_new0 (TrayApplet, 1);
 
    // check if the updates icon should be displayed
    if (in_admin_group() || FORCE_START) {
-      trayapplet_create(un->update, "software-update-available");
+      trayapplet_create(un->update, un, "software-update-available");
       update_tray_icon_init(un->update);
    } else 
       un->update = NULL;
 
    /* update hook icon*/
    un->hook = g_new0 (TrayApplet, 1);
-   trayapplet_create(un->hook, "hook-notifier");
+   trayapplet_create(un->hook, un, "hook-notifier");
    hook_tray_icon_init(un->hook);
 
    /* reboot required icon */
    un->reboot = g_new0 (TrayApplet, 1);
-   trayapplet_create(un->reboot, "un-reboot");
+   trayapplet_create(un->reboot, un, "un-reboot");
    reboot_tray_icon_init(un->reboot);
 
    /* crashreport detected icon */
    un->crashreport = g_new0 (TrayApplet, 1);
-   trayapplet_create(un->crashreport, "apport");
+   trayapplet_create(un->crashreport, un, "apport");
    crashreport_tray_icon_init(un->crashreport);
 
    return FALSE; // for the tray_destroyed_cb
 }
 
-gboolean
-system_user()
+static gboolean
+system_user(UpgradeNotifier *un)
 {
    /* do not start for system users, e.g. the guest session 
     * (see /usr/share/gdm/guest-session/guest-session-setup.sh)
     */
    int end_system_uid = 500;
-   GSettings *settings = g_settings_new(SETTINGS_SCHEMA);
-   int i = g_settings_get_int(settings, SETTINGS_KEY_END_SYSTEM_UIDS);
+   int i = g_settings_get_int(un->settings, SETTINGS_KEY_END_SYSTEM_UIDS);
    if (i>0)
       end_system_uid = i;
-   g_object_unref(settings);
 
    uid_t uid = getuid();
    if (uid < end_system_uid)
@@ -544,7 +517,7 @@ user_in_group(const char* grpname)
 }
 
 gboolean
-in_admin_group()
+in_admin_group(void)
 {
    int is_admin, is_sudo;
 
@@ -556,6 +529,14 @@ in_admin_group()
 }
 
 
+static gboolean
+sigint_cb (gpointer user_data __attribute__ ((__unused__)))
+{
+	gtk_main_quit ();
+	return FALSE;
+}
+
+
 static GOptionEntry entries[] = 
 {
    { "debug-hooks", 0, 0, G_OPTION_ARG_NONE, &HOOK_DEBUG, "Enable hooks debugging"},
@@ -565,7 +546,7 @@ static GOptionEntry entries[] =
    { "debug-new-release-check", 0, 0, G_OPTION_ARG_NONE, &RELEASE_DEBUG, "Enable new release check debugging"},
    { "debug-misc", 0, 0, G_OPTION_ARG_NONE, &MISC_DEBUG, "Enable uncategorized debug prints"},
    { "force", 0, 0, G_OPTION_ARG_NONE, &FORCE_START, "Force start even if the user is not in the admin group"},
-   { "force-use-gksu", 0, 0, G_OPTION_ARG_NONE, &FORCE_GKSU, "Force running all commands (update-manager, synaptic) with gksu" },
+   { "force-use-pkexec", 0, 0, G_OPTION_ARG_NONE, &FORCE_PKEXEC, "Force running all commands (update-manager, synaptic) with pkexec" },
    { "startup-delay", 0, 0, G_OPTION_ARG_INT, &STARTUP_DELAY, "Delay startup by given amount of seconds" },
    { "devel-release", 0, 0, G_OPTION_ARG_NONE, &DEVEL_RELEASE, "Make the release checker check for a new development release" },
    { "force-release-check", 0, 0, G_OPTION_ARG_NONE, &FORCE_RELEASE_CHECK, "Force release check" },
@@ -592,6 +573,10 @@ main (int argc, char **argv)
         bind_textdomain_codeset(PACKAGE, "UTF-8");
         textdomain(PACKAGE);
 
+	if (HOOK_DEBUG || UPDATE_DEBUG || INOTIFY_DEBUG ||
+	    UEVENT_DEBUG || RELEASE_DEBUG || MISC_DEBUG)
+		g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
+
 	// setup a custom debug log handler
 	g_log_set_handler ("inotify", G_LOG_LEVEL_DEBUG,
 			   debug_log_handler, NULL);
@@ -611,28 +596,26 @@ main (int argc, char **argv)
 
 	//g_print("starting update-notifier\n");
 
+	/* Create the UpgradeNotifier object */
+	un = g_new0 (UpgradeNotifier, 1);
+	un->settings = g_settings_new(SETTINGS_SCHEMA);
+
 	// do not run as system user (e.g. guest user)
-	if (system_user() && !FORCE_START) {
-	   g_warning("not starting for system user\n");
+	if (system_user(un) && !FORCE_START) {
+	   g_warning("not starting for system user");
 	   exit(0);
 	}
 
 	// do not run as system user (e.g. guest user)
-	if (FORCE_RELEASE_CHECK) {
-	   GSettings *settings = g_settings_new(SETTINGS_SCHEMA);
-	   g_settings_reset(settings, SETTINGS_KEY_LAST_RELEASE_CHECK);
-	   g_object_unref(settings);
-	}
+	if (FORCE_RELEASE_CHECK)
+	   g_settings_reset(un->settings, SETTINGS_KEY_LAST_RELEASE_CHECK);
 
 	// check if it is running already
 	if (!up_get_clipboard ()) {
-	   g_warning ("already running?\n");
+	   g_warning ("already running?");
 	   return 1;
 	}
 
-	/* Create the UpgradeNotifier object */
-	un = g_new0 (UpgradeNotifier, 1);
-
 	// check for update-notifier dir and create if needed
 	gchar *dirname = g_strdup_printf("%s/update-notifier",
 					 g_get_user_config_dir());
@@ -647,22 +630,24 @@ main (int argc, char **argv)
 
         // initial check for avahi
         avahi_disabled_check();
-	
+
 	/* setup inserted-cdroms checker */
 	if(!cdroms_init(un)) {
 	   g_warning("initializing cdroms failed");
 	}
 
 	// init release checker
-	release_checker_init();
+	release_checker_init(un);
 
 	// init uevent monitoring (missing firmware, etc.)
 	uevent_init();
 
 	// init gio file monitoring
 	monitor_init (un);
-	
+
 	/* Start the main gtk loop */
+	g_unix_signal_add_full (G_PRIORITY_DEFAULT, SIGINT, sigint_cb,
+				NULL, NULL);
 	gtk_main ();
 
 	return 0;
