@@ -21,7 +21,7 @@
 
 #define UPGRADE_CHECKER PACKAGE_LIB_DIR"/update-notifier/apt-check"
 
-// command, description, desktopfile, needs_gksu
+// command, description, desktopfile, needs_pkexec
 const char* actions[][4] = {
    { "/usr/lib/update-notifier/backend_helper.py show_updates",
      N_("Show updates"), 
@@ -35,7 +35,7 @@ const char* actions[][4] = {
      N_("Check for updates"), "/usr/share/applications/synaptic.desktop", 
      GINT_TO_POINTER(FALSE) },
    { "/usr/lib/update-notifier/backend_helper.py start_packagemanager",
-     "/usr/share/applications/synaptic.desktop", 
+     N_("Start package manager"), "/usr/share/applications/synaptic.desktop",
      GINT_TO_POINTER(FALSE)},
    { NULL, NULL, NULL }
 };
@@ -55,6 +55,7 @@ g_debug_update(const char *msg, ...)
    va_end(va);
 }
 
+#ifndef HAVE_APP_INDICATOR
 static gboolean
 gtk_status_icon_click_activate_cb (GtkWidget *widget, 
 	     TrayApplet *ta)
@@ -70,8 +71,9 @@ gtk_status_icon_click_activate_cb (GtkWidget *widget,
    last_action = time(NULL);
    return TRUE;
 }
+#endif
 
-void
+static void
 update_trayicon_update_tooltip (TrayApplet *ta, int num_upgrades)
 {
    //g_print("update_tooltip: %p %p %p\n", ta, ta->tooltip, ta->eventbox);
@@ -86,18 +88,14 @@ update_trayicon_update_tooltip (TrayApplet *ta, int num_upgrades)
 }
 
 
-void 
-cb_action(GObject *self, void *user_data)
+static void 
+cb_action(GObject *self, void *user_data __attribute__ ((__unused__)))
 {
-   TrayApplet *ta = user_data;
-	
    int i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(self), "action"));
    invoke (actions[i][0], _(actions[i][2]), (long)actions[i][3]);
-
-   UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
 }
 
-void 
+static void 
 cb_preferences(GObject *self, void *user_data)
 {
    invoke("/usr/bin/software-properties-gtk",
@@ -105,26 +103,25 @@ cb_preferences(GObject *self, void *user_data)
           FALSE);    
 }
 
-void 
+static void 
 cb_toggled_show_notifications(GObject *self, void *data)
 {
       TrayApplet *ta = (TrayApplet*)data;
       UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
 
       gboolean b = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(self));
-      g_settings_set_boolean(priv->settings,
+      g_settings_set_boolean(ta->un->settings,
                              SETTINGS_KEY_NO_UPDATE_NOTIFICATIONS, !b);
       
 
-      NotifyNotification *n = priv->active_notification;
-      if(n != NULL) {
-	 notify_notification_close(n, NULL);
-	 priv->active_notification = NULL;
+      if(priv->active_notification != NULL) {
+	 notify_notification_close(priv->active_notification, NULL);
+	 g_clear_object(&priv->active_notification);
       }
 }
 
 
-void
+static void
 update_trayicon_create_menu(TrayApplet *ta)
 {
 	GtkWidget *menuitem;
@@ -147,8 +144,7 @@ update_trayicon_create_menu(TrayApplet *ta)
 	gtk_menu_shell_append (GTK_MENU_SHELL (ta->menu), menuitem);
 
 	menuitem = gtk_check_menu_item_new_with_label(_("Show notifications"));
-	UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
-	gboolean b = g_settings_get_boolean(priv->settings,
+	gboolean b = g_settings_get_boolean(ta->un->settings,
 	                                    SETTINGS_KEY_NO_UPDATE_NOTIFICATIONS);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), !b);
 	gtk_menu_shell_append (GTK_MENU_SHELL (ta->menu), menuitem);
@@ -169,7 +165,7 @@ update_trayicon_create_menu(TrayApplet *ta)
 void 
 update_apt_is_running(TrayApplet *ta, gboolean is_running)
 {
-   g_debug_update("update_apt_is_running: %i\n",is_running);
+   g_debug_update("update_apt_is_running: %i",is_running);
    if(ta == NULL)
       return;
 
@@ -179,7 +175,7 @@ update_apt_is_running(TrayApplet *ta, gboolean is_running)
 
    // if the user wants auto-launch mode, do not show the icon
    // the auto launch stuff will do its magic later
-   if(g_settings_get_boolean(priv->settings, SETTINGS_KEY_AUTO_LAUNCH))
+   if(g_settings_get_boolean(ta->un->settings, SETTINGS_KEY_AUTO_LAUNCH))
       return;
 
    if(is_running) {
@@ -212,7 +208,7 @@ update_apt_is_running(TrayApplet *ta, gboolean is_running)
 
       if(priv->active_notification != NULL) {
 	 notify_notification_close(priv->active_notification, NULL);
-	 priv->active_notification = NULL;
+	 g_clear_object(&priv->active_notification);
       }
    } else {
       tray_applet_ui_set_icon(ta, ta->name);
@@ -226,7 +222,7 @@ show_notification(gpointer user_data)
    TrayApplet *ta = (TrayApplet *)user_data;
    UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
 
-   // apt is runing, no point in showing a notification
+   // apt is running, no point in showing a notification
    if(priv->apt_is_running) 
       return TRUE;
 
@@ -259,6 +255,8 @@ show_notification(gpointer user_data)
    notify_notification_set_timeout (n, 60*1000);
    notify_notification_show(n, NULL);
    // save the notification handle
+   if (priv->active_notification)
+      g_object_unref(priv->active_notification);
    priv->active_notification = n;
 
    // remove this from the timeout now
@@ -266,7 +264,7 @@ show_notification(gpointer user_data)
    return FALSE;
 }
 
-void 
+static void 
 show_error(TrayApplet *ta, gchar *error_str)
 {
    tray_applet_ui_set_tooltip_text(ta, error_str);
@@ -302,7 +300,7 @@ outdated_nag(TrayApplet *ta)
 //
 // if uncertain, return FALSE 
 static gboolean
-dpkg_lock_is_taken ()
+dpkg_lock_is_taken (void)
 {
   GError *error;
   GVariant *answer;
@@ -319,7 +317,7 @@ dpkg_lock_is_taken ()
                                          NULL, /* GCancellable */
                                          &error);
   if (proxy == NULL) {
-    g_debug_update ("Failed to open connection to bus: %s\n", error->message);
+    g_debug_update ("Failed to open connection to bus: %s", error->message);
     g_error_free (error);
     return FALSE;
   }
@@ -330,13 +328,13 @@ dpkg_lock_is_taken ()
   g_object_unref (proxy);
 
   if (answer == NULL) {
-    g_debug_update ("error during dbus call: %s\n", error->message);
+    g_debug_update ("error during dbus call: %s", error->message);
     g_error_free (error);
     return FALSE;
   }
 
   if (g_strcmp0 (g_variant_get_type_string (answer), "(b)") != 0) {
-    g_debug_update ("SystemService answer in unexpected format: %s\n",
+    g_debug_update ("SystemService answer in unexpected format: %s",
                     g_variant_get_type_string (answer));
     g_variant_unref (answer);
     return FALSE;
@@ -352,7 +350,7 @@ dpkg_lock_is_taken ()
 #if 0
 // ask devicekit.Power if we run on battery
 static gboolean
-running_on_battery ()
+running_on_battery (void)
 {
    DBusGConnection *connection;
    GError *error;
@@ -362,7 +360,7 @@ running_on_battery ()
    error = NULL;
    connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
    if (connection == NULL) {
-      g_debug_update ("Failed to open connection to bus: %s\n", error->message);
+      g_debug_update ("Failed to open connection to bus: %s", error->message);
       g_error_free (error);
       return FALSE;
    }
@@ -378,7 +376,7 @@ running_on_battery ()
 			  G_TYPE_INVALID,
                           G_TYPE_BOOLEAN, &on_battery, 
 			  G_TYPE_INVALID)) {
-     g_debug_update ("failed Get dbus call: %s\n", 
+     g_debug_update ("failed Get dbus call: %s", 
 		     error?error->message:"no error given");
      if(error)
 	g_error_free (error);
@@ -390,7 +388,7 @@ running_on_battery ()
 #endif
 
 static gboolean
-security_updates_are_installed_unattended()
+security_updates_are_installed_unattended(void)
 {
    int ret;
    GError *error = NULL;
@@ -407,7 +405,7 @@ security_updates_are_installed_unattended()
 
       if (g_spawn_sync("/", cmd, NULL, 0, NULL, NULL, 
 		       NULL, NULL, &ret, &error)) {
-	 g_debug("--security-updates-unattended: %i\n", WEXITSTATUS(ret));
+	 g_debug("--security-updates-unattended: %i", WEXITSTATUS(ret));
 	 if( WEXITSTATUS(ret) > 0 )
 	    return TRUE;
       } else if (error) {
@@ -433,7 +431,7 @@ auto_launch_security_now(UpdateTrayAppletPrivate *priv,
    // security updates, but already launched recently
    if ((last_launch + AUTOLAUNCH_MINIMAL_SECURITY_INTERVAL) > now) {
       g_debug_update("security updates, but update-manager was launched "
-		     "within the AUTOLAUNCH_MINIMAL_SECURITY_INTERVAL\n");
+		     "within the AUTOLAUNCH_MINIMAL_SECURITY_INTERVAL");
       return FALSE;
    }
    
@@ -446,11 +444,25 @@ auto_launch_security_now(UpdateTrayAppletPrivate *priv,
    return TRUE;
 }
 
+// Like ctime, but without the annoying trailing newline.  Caller must
+// g_free the result.
+static gchar *
+readable_timestamp(time_t time)
+{
+   GDateTime *datetime;
+   gchar *formatted;
+
+   datetime = g_date_time_new_from_unix_local (time);
+   formatted = g_date_time_format (datetime, "%a %b %e %T %Y");
+   g_date_time_unref (datetime);
+   return formatted;
+}
+
 // check the logs of dpkg and apt for timestamps, the idea
 // is to not auto launch if dpkg/apt were run manually by the
 // user
 static time_t
-newest_log_file_timestamp()
+newest_log_file_timestamp(void)
 {
    struct stat buf;
    int i, j;
@@ -464,26 +476,37 @@ newest_log_file_timestamp()
    {
       glob_t pglob;
       if(glob(log_glob[i], 0, NULL, &pglob) != 0) {
-	 g_warning("error from glob %s\n", log_glob[i]);
+	 g_warning("error from glob %s", log_glob[i]);
 	 continue;
       }
       for(j=0;j < pglob.gl_pathc; j++) {
-
 	 const char *log = pglob.gl_pathv[j];
+	 time_t mtime, bctime;
+	 gchar *timestamp;
+
 	 if(g_stat(log, &buf) <0) {
-	    g_warning("can't stat %s\n", log);
+	    g_warning("can't stat %s", log);
 	    continue;
 	 }
 	 if(buf.st_size == 0) {
-	    g_warning("log file empty (logrotate?) %s\n", log);
+	    g_warning("log file empty (logrotate?) %s", log);
 	    continue;
 	 }
-	 time_t mtime = buf.st_mtime;
-	 g_debug_update ("mtime from %s: %i (%s)\n", log, mtime, ctime(&mtime));
-	 time_t bctime = buf.st_ctime;
-	 g_debug_update ("ctime from %s: %i (%s)\n", log, bctime, ctime(&bctime));
+
+	 mtime = buf.st_mtime;
+	 timestamp = readable_timestamp (mtime);
+	 g_debug_update ("mtime from %s: %i (%s)", log, mtime, timestamp);
+	 g_free (timestamp);
+
+	 bctime = buf.st_ctime;
+	 timestamp = readable_timestamp (bctime);
+	 g_debug_update ("ctime from %s: %i (%s)", log, bctime, timestamp);
+	 g_free (timestamp);
+
 	 newest_log_stamp = MAX(MAX(mtime, bctime), newest_log_stamp);
-	 g_debug_update ("last_launch from %s: %i (%s)\n", log, newest_log_stamp, ctime(&newest_log_stamp));
+	 timestamp = readable_timestamp (newest_log_stamp);
+	 g_debug_update ("last_launch from %s: %i (%s)", log, newest_log_stamp, timestamp);
+	 g_free (timestamp);
       }
       globfree(&pglob);
    }
@@ -494,8 +517,9 @@ newest_log_file_timestamp()
 // time to launch again and if the dpkg lock is currently 
 // not taken
 static gboolean 
-auto_launch_now (UpdateTrayAppletPrivate *priv)
+auto_launch_now (TrayApplet *ta)
 {
+   UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
    int interval_days = 0;
    time_t last_launch = 0;
 
@@ -510,9 +534,9 @@ auto_launch_now (UpdateTrayAppletPrivate *priv)
 
    // when checking for regular updates honor the 
    // regular-auto-launch-interval
-   interval_days = g_settings_get_int(priv->settings,
+   interval_days = g_settings_get_int(ta->un->settings,
 					SETTINGS_KEY_AUTO_LAUNCH_INTERVAL);
-   g_debug_update ("interval_days: %i\n", interval_days);
+   g_debug_update ("interval_days: %i", interval_days);
 
    if (interval_days <= 0) 
       return TRUE;
@@ -521,7 +545,7 @@ auto_launch_now (UpdateTrayAppletPrivate *priv)
    GSettings *um_settings = g_settings_new(SETTINGS_UM_SCHEMA);
    last_launch = g_settings_get_int(um_settings,
                                     SETTINGS_UM_KEY_LAST_LAUNCH);
-   g_debug_update ("last_launch: %i (%s)\n", last_launch, ctime(&last_launch));
+   g_debug_update ("last_launch: %i (%s)", last_launch, ctime(&last_launch));
    g_object_unref(um_settings);
 
    time_t now = time(NULL);
@@ -546,7 +570,7 @@ update_check (TrayApplet *ta)
    if(ta == NULL)
       return FALSE;
 
-   g_debug_update ("update_check()\n");
+   g_debug_update ("update_check()");
    UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
 
    priv->num_upgrades = 0;
@@ -627,7 +651,7 @@ update_check (TrayApplet *ta)
       tray_applet_ui_set_visible (ta, FALSE);
       if(priv->active_notification != NULL) {
 	 notify_notification_close(priv->active_notification, NULL);
-	 priv->active_notification = NULL; 
+	 g_clear_object(&priv->active_notification);
       }
       return TRUE;
    } 
@@ -643,10 +667,10 @@ update_check (TrayApplet *ta)
 
    // check if the user wants to see the icon or launch
    // the default action
-   if(g_settings_get_boolean(priv->settings, SETTINGS_KEY_AUTO_LAUNCH))
+   if(g_settings_get_boolean(ta->un->settings, SETTINGS_KEY_AUTO_LAUNCH))
    {
       tray_applet_ui_set_visible(ta, FALSE);
-      if (auto_launch_now(priv)) 
+      if (auto_launch_now(ta)) 
       {
 	 g_spawn_command_line_async("nice ionice -c3 update-manager "
 				    "--no-update --no-focus-on-map", NULL);
@@ -662,7 +686,7 @@ update_check (TrayApplet *ta)
    tray_applet_ui_set_visible(ta, TRUE);
    
    // the user does not no notification messages
-   if(g_settings_get_boolean(priv->settings,
+   if(g_settings_get_boolean(ta->un->settings,
                              SETTINGS_KEY_NO_UPDATE_NOTIFICATIONS))
       return TRUE;
 
@@ -679,11 +703,14 @@ update_tray_icon_init(TrayApplet *ta)
 {
    // create the private data struct
    UpdateTrayAppletPrivate *priv = g_new0(UpdateTrayAppletPrivate, 1);
-   priv->settings = g_settings_new(SETTINGS_SCHEMA);
    priv->apt_is_running = FALSE;
    priv->active_notification = NULL;
    ta->user_data = priv;
 
+   // create the indicator/icon immediately; we use this enough that it
+   // probably isn't worth trying to save memory by avoiding it
+   tray_applet_ui_ensure (ta);
+
 #ifndef HAVE_APP_INDICATOR
    // the default action for the gtk_status_icon
    g_signal_connect (G_OBJECT(ta->tray_icon),
